@@ -57,8 +57,6 @@
 #include "libmal.h"
 #include "config.h"
 
-
-
 #ifdef _DEBUG
 #define debug_print(x) printf("%s\n", (x));
 #else
@@ -180,8 +178,13 @@ void syncInfoFree(PalmSyncInfo * pInfo) {
         if (NULL != pInfo->userConfig)
             AGUserConfigFree(pInfo->userConfig);
 
+#ifdef PILOT_LINK_0_12
+        if (NULL != pInfo->pi_buf)
+            pi_buffer_free(pInfo->pi_buf);
+#else
         if (NULL != pInfo->pilot_buffer)
             free(pInfo->pilot_buffer);
+#endif
 
         if (NULL != pInfo->commandProcessor)
             AGCommandProcessorFree(pInfo->commandProcessor);
@@ -202,10 +205,16 @@ PalmSyncInfo *syncInfoNew() {
 
         bzero(pInfo, sizeof(PalmSyncInfo));
 
+#ifdef PILOT_LINK_0_12
+        pInfo->pi_buf = pi_buffer_new(pbs);
+        if (NULL == pInfo->pi_buf)
+            goto fail;
+#else
         pInfo->pilot_buffer_size    = pbs;
         pInfo->pilot_buffer         = (uint8 *)malloc(pbs);
         if (NULL == pInfo->pilot_buffer)
             goto fail;
+#endif
 
         /* Allocate the platform calls record. */
         pInfo->platform = (AGPlatformCalls *)malloc(sizeof(AGPlatformCalls));
@@ -257,8 +266,12 @@ int16 readInt16(uint8 * p)
 static void 
 readAndUseDeviceInfoDatabase(AGDeviceInfo *devInfo,
                              int sd,
+#ifdef PILOT_LINK_0_12
+                             pi_buffer_t *pi_buf)
+#else
                              uint8 *dev_db_info_buffer,
                              uint32 dev_db_info_buffer_size)
+#endif
 {
     int database_id = 0;
     long result;
@@ -291,7 +304,7 @@ readAndUseDeviceInfoDatabase(AGDeviceInfo *devInfo,
 
 #ifdef PILOT_LINK_0_12
         rc = dlp_ReadRecordByIndex(sd, database_id, 0,
-                                   (void *)dev_db_info_buffer,
+                                   pi_buf,
                                    &id, &attr, &cat);
 #else
         rc = dlp_ReadRecordByIndex(sd, database_id, 0,
@@ -301,7 +314,11 @@ readAndUseDeviceInfoDatabase(AGDeviceInfo *devInfo,
 #endif
 
         if (rc >= 0) {
+#ifdef PILOT_LINK_0_12
+            uint8 *p = pi_buf->data;
+#else
             uint8 *p = dev_db_info_buffer;
+#endif
             /*int16 dev_db_info_version = readInt16(p);*/
             /* make gcc happy */
             readInt16(p);
@@ -380,7 +397,6 @@ static int readDeviceInfo(PalmSyncInfo *pInfo) {
     build = (((sysInfo.romVersion >> 8) & 0xf) * 10) + 
         (((sysInfo.romVersion >> 4) & 0xf) * 10)+ (sysInfo.romVersion  & 0xf);
     
-
     snprintf(osverstring, 24, "%d.%d", majorVersion, minorVersion);
 
     if (verbose) {
@@ -410,8 +426,12 @@ static int readDeviceInfo(PalmSyncInfo *pInfo) {
 
     readAndUseDeviceInfoDatabase(devInfo,
                                  pInfo->sd,
+#ifdef PILOT_LINK_0_12
+                                 pInfo->pi_buf);
+#else
                                  pInfo->pilot_buffer,
                                  pInfo->pilot_buffer_size);
+#endif
 
     /* Override the color depth if the user wants low res images. */
     if (pInfo->lowres) {
@@ -611,15 +631,20 @@ static int32
 readDeviceUserConfig32(int sd, int userConfigDBHandle, AGUserConfig **deviceUserConfig)
 {
     recordid_t id;
+#ifdef PILOT_LINK_0_12
+    pi_buffer_t *pi_buf;
+#else
     int bufferSize = BUFFERSIZE;
+    uint8 buffer[BUFFERSIZE];
+#endif
     int attr = 0;
     int cat  = 0;
     int rc;
-    uint8 buffer[BUFFERSIZE];
     AGBufferReader * r = NULL;
 
 #ifdef PILOT_LINK_0_12
-    rc = dlp_ReadRecordByIndex(sd, userConfigDBHandle, 0, (void *)buffer,
+    pi_buf = pi_buffer_new(BUFFERSIZE);
+    rc = dlp_ReadRecordByIndex(sd, userConfigDBHandle, 0, pi_buf,
                                &id, &attr, &cat);
 #else
     rc = dlp_ReadRecordByIndex(sd, userConfigDBHandle, 0, (void *)buffer,
@@ -635,18 +660,32 @@ readDeviceUserConfig32(int sd, int userConfigDBHandle, AGUserConfig **deviceUser
          * It would be nice to get the original error somehow...
          */
         lm_errno = LM_READ_RECORD;
+#ifdef PILOT_LINK_0_12
+        pi_buffer_free(pi_buf);
+#endif
         return 0;
     }
     
+#ifdef PILOT_LINK_0_12
+    r = AGBufferReaderNew(pi_buf->data);
+#else
     r = AGBufferReaderNew(buffer);
+#endif
     if (r) {
         *deviceUserConfig = AGUserConfigNew();
         AGUserConfigReadData(*deviceUserConfig, (AGReader*)r);
         AGBufferReaderFree(r);
+#ifdef PILOT_LINK_0_12
+        pi_buffer_free(pi_buf);
+#endif
         return id;
     }
-    else
+    else {
+#ifdef PILOT_LINK_0_12
+        pi_buffer_free(pi_buf);
+#endif
         return 0;
+    }
 }
 #define BUFFERSIZE 0xFFFF
 /*---------------------------------------------------------------------------*/
@@ -654,15 +693,20 @@ static int32 readDeviceUserConfig31(int sd, int userConfigDBHandle,
                                     AGUserConfig **deviceUserConfig)
 {
     recordid_t id;
+#ifdef PILOT_LINK_0_12
+    pi_buffer_t *pi_buf;
+#else
     int bufferSize = BUFFERSIZE;
+    uint8 buffer[BUFFERSIZE];
+#endif
     int attr = 0;
     int cat  = 0;
     int rc;
-    uint8 buffer[BUFFERSIZE];
     AGBufferReader * r = NULL;
 
 #ifdef PILOT_LINK_0_12
-    rc = dlp_ReadRecordByIndex(sd, userConfigDBHandle, 0, (void *)buffer,
+    pi_buf = pi_buffer_new(BUFFERSIZE);
+    rc = dlp_ReadRecordByIndex(sd, userConfigDBHandle, 0, pi_buf,
                                &id, &attr, &cat);
 #else
     rc = dlp_ReadRecordByIndex(sd, userConfigDBHandle, 0, (void *)buffer,
@@ -678,17 +722,32 @@ static int32 readDeviceUserConfig31(int sd, int userConfigDBHandle,
          * It would be nice to get the original error somehow...
          */
         lm_errno = LM_READ_RECORD;
+#ifdef PILOT_LINK_0_12
+        pi_buffer_free(pi_buf);
+#endif
         return 0;
     }
     
+#ifdef PILOT_LINK_0_12
+    r = AGBufferReaderNew(pi_buf->data);
+#else
     r = AGBufferReaderNew(buffer);
+#endif
     if (r) {
         *deviceUserConfig = AGUserConfigNew();
         MAL31ReadUserData(*deviceUserConfig, (AGReader*)r);
         AGBufferReaderFree(r);
+#ifdef PILOT_LINK_0_12
+        pi_buffer_free(pi_buf);
+#endif
         return id;
-    } else
+    }
+    else {
+#ifdef PILOT_LINK_0_12
+        pi_buffer_free(pi_buf);
+#endif
         return 0;
+    }
 }
 /*---------------------------------------------------------------------------*/
 static int32 readDeviceUserConfig(int sd, int userConfigDBHandle,
@@ -711,14 +770,21 @@ static void writeDeviceUserConfig(int sd,
 {
 
     recordid_t id;
+#ifndef PILOT_LINK_0_12
     int bufferSize = BUFFERSIZE;
+    uint8 buffer[BUFFERSIZE];
+#endif
     int attr = 0;
     int cat  = 0;
-    uint8 buffer[BUFFERSIZE];
     AGBufferWriter * w = NULL;
     w = AGBufferWriterNew(0);
     if (w) {
         long result;
+#ifdef PILOT_LINK_0_12
+        pi_buffer_t *pi_buf;
+
+        pi_buf = pi_buffer_new(BUFFERSIZE);
+#endif
         
         if (threeone) {
             MAL31WriteUserData(deviceUserConfig, (AGWriter*)w);
@@ -727,7 +793,7 @@ static void writeDeviceUserConfig(int sd,
         }
 
 #ifdef PILOT_LINK_0_12
-        result = dlp_ReadRecordByIndex(sd, userConfigDBHandle, 0, (void *)buffer,
+        result = dlp_ReadRecordByIndex(sd, userConfigDBHandle, 0, pi_buf,
                                        &id, &attr, &cat);
 #else
         result = dlp_ReadRecordByIndex(sd, userConfigDBHandle, 0, (void *)buffer,
@@ -743,6 +809,9 @@ static void writeDeviceUserConfig(int sd,
                                   &id);
         AGBufferWriterFree(w);
 
+#ifdef PILOT_LINK_0_12
+        pi_buffer_free(pi_buf);
+#endif
     }
 }
 
@@ -1041,16 +1110,18 @@ getRecordBase(PalmSyncInfo *pInfo, AGBool modonly,  AGRecord **record,
     int32 result;
     int att = 0;
     int cat = 0;
+#ifndef PILOT_LINK_0_12
     int size = pInfo->pilot_buffer_size;
+#endif
     int idx   = pInfo->pilot_RecIndex++;
 
     result = (modonly) ?
 #ifdef PILOT_LINK_0_12
-        dlp_ReadNextModifiedRec (pInfo->sd, pInfo->pilot_rHandle, pInfo->pilot_buffer,
+        dlp_ReadNextModifiedRec (pInfo->sd, pInfo->pilot_rHandle, pInfo->pi_buf,
                                  &pInfo->id, &idx, &att, &cat)
         :
         dlp_ReadRecordByIndex(pInfo->sd, pInfo->pilot_rHandle, idx,
-                              (void *)pInfo->pilot_buffer,
+                              (void *)pInfo->pi_buf,
                               &pInfo->id, &att, &cat);
 #else
         dlp_ReadNextModifiedRec (pInfo->sd, pInfo->pilot_rHandle, pInfo->pilot_buffer,
@@ -1076,7 +1147,11 @@ getRecordBase(PalmSyncInfo *pInfo, AGBool modonly,  AGRecord **record,
     }
     pInfo->record = AGRecordInit(pInfo->record, pInfo->id,
                                  AGPalmPilotAttribsToMALMod((uint8)att),
+#ifdef PILOT_LINK_0_12
+                                 pInfo->pi_buf->used, pInfo->pi_buf->data, 0, NULL);
+#else
                                  size, pInfo->pilot_buffer, 0, NULL);
+#endif
 
     *record = pInfo->record;
     return AGCLIENT_CONTINUE;
@@ -1246,7 +1321,7 @@ static AGBool doClientProcessorLoop(PalmSyncInfo *pInfo, AGNetCtx *ctx) {
             pInfo->clientProcessor->version_info = 
                 malloc(sizeof(AGExpansionVersion));
             if (NULL == pInfo->clientProcessor->version_info) 
-                exit(-1);
+                return -1;
 
             fill_in_versioninfo(pInfo->sd, pInfo->clientProcessor->version_info);
 
@@ -1409,14 +1484,18 @@ fill_in_versioninfo(int sd, AGExpansionVersion *vi)
         int cat  = 0;
         recordid_t id;
         int rc;
+#ifdef PILOT_LINK_0_12
+        pi_buffer_t *version_buffer = pi_buffer_new(0x1000);
+#else
         int version_buffer_size = 0x1000;
         uint8 version_buffer[version_buffer_size];
+#endif
 
 
 #ifdef PILOT_LINK_0_12
         rc = dlp_ReadRecordByIndex(sd, database_id, 
                                    0, 
-                                   (void *)version_buffer, 
+                                   version_buffer, 
                                    &id, 
                                    &attr, 
                                    &cat);
@@ -1432,9 +1511,15 @@ fill_in_versioninfo(int sd, AGExpansionVersion *vi)
 
         if (rc >= 0) {
             ClientVersion cv;
+#ifdef PILOT_LINK_0_12
+            ClientVersionReadData(&cv,
+                                  version_buffer->used,
+                                  version_buffer->data);
+#else
             ClientVersionReadData(&cv,
                                   version_buffer_size,
                                   version_buffer);
+#endif
             vi->device_major = cv.major;
             vi->device_minor = cv.minor;
             vi->device_build = cv.buildno;
@@ -1443,10 +1528,12 @@ fill_in_versioninfo(int sd, AGExpansionVersion *vi)
             vi->conn_agent_minor = cv.minor;
             vi->conn_agent_build = cv.buildno;
         }
+
+#ifdef PILOT_LINK_0_12
+        pi_buffer_free(version_buffer);
+#endif
         dlp_CloseDB(sd, database_id);
-
     }
-
 }
 
 int malsync (PalmSyncInfo *pInfo) {
